From bb42057055651a78b4cccc9d4b6e9deba8dcf43a Mon Sep 17 00:00:00 2001
From: Changhui Liu <liuchanghui@linuxdeepin.com>
Date: Wed, 10 Dec 2014 14:28:51 +0800
Subject: gdiplus: fix GdipFillRectangleI no effect on memory DC whose window
 origin point has changed
To: wine-patches <wine-patches@winehq.org>
Reply-To: wine-devel <wine-devel@winehq.org>

---
diff --git a/dlls/gdiplus/graphics.c b/dlls/gdiplus/graphics.c
index 840d866..2ef3acd 100644
--- a/dlls/gdiplus/graphics.c
+++ b/dlls/gdiplus/graphics.c
@@ -495,7 +495,23 @@ static GpStatus alpha_blend_pixels_hrgn(GpGraphics *graphics, INT dst_x, INT dst
             ExtSelectClipRgn(graphics->hdc, hrgn, RGN_AND);
 
         if (hregion)
+        {
+            /*The user maybe has called SetWindowOrgEx or like API to change the window origin point. 
+            And the hregion use the new changed origin point whose value maybe is not zero,
+            but the ExtSelectClipRgn use the origin point whose value is zero.
+            */
+            POINT pt = {0, 0}; 
+            LPtoDP(graphics->hdc, &pt, 1); 
+            if (pt.x!=0 || pt.y!=0)
+            { 
+                if (GetMapMode(graphics->hdc) == MM_TEXT)
+                {
+                    OffsetRgn (hregion, pt.x, pt.y);
+                }
+                /*FIXME: if (GetMapMode(graphics->hdc) != MM_TEXT) ...*/
+            } 
             ExtSelectClipRgn(graphics->hdc, hregion, RGN_AND);
+        }
 
         stat = alpha_blend_hdc_pixels(graphics, dst_x, dst_y, src, src_width,
             src_height, src_stride);
@@ -2029,8 +2045,7 @@ static GpStatus get_graphics_bounds(GpGraphics* graphics, GpRectF* rect)
         rect->Height = GetDeviceCaps(graphics->hdc, VERTRES);
     }
 
-    if (graphics->hdc &&
-        (GetMapMode(graphics->hdc) != MM_TEXT || GetGraphicsMode(graphics->hdc) != GM_COMPATIBLE))
+    if (graphics->hdc)
     {
         POINT points[2];
 
diff --git a/dlls/gdiplus/tests/graphics.c b/dlls/gdiplus/tests/graphics.c
index bd227dc..afdd52a 100644
--- a/dlls/gdiplus/tests/graphics.c
+++ b/dlls/gdiplus/tests/graphics.c
@@ -5502,6 +5502,117 @@ static void test_clipping_2(void)
     DeleteDC(hdc);
 }
 
+static BOOL IsFillOK(HDC hdc, HBITMAP hbmp, UINT width, UINT height)
+{
+    BITMAPINFOHEADER bi;
+    UINT x = 0;
+    UINT y = 0;
+    UINT lines = 0;
+    BYTE *buffer = (BYTE *)GdipAlloc(width*height*4);
+    DWORD *pixel = (DWORD *)buffer;
+
+    bi.biSize = sizeof(BITMAPINFOHEADER);
+    bi.biWidth = width;
+    bi.biHeight = height;
+    bi.biPlanes = 1;
+    bi.biBitCount = 32;
+    bi.biCompression = BI_RGB;
+    bi.biSizeImage = 0;
+    bi.biXPelsPerMeter = 0;
+    bi.biYPelsPerMeter = 0;
+    bi.biClrUsed = 0;
+    bi.biClrImportant = 0;
+
+    lines = GetDIBits(hdc, hbmp, 0, height, pixel, (BITMAPINFO *)&bi, DIB_RGB_COLORS);
+    if (lines != height )
+    {
+        trace("%p,%d -> %d,%d\n", pixel, height, lines, GetLastError());
+        return 0;
+    }
+
+    for(y = 0; y<height; ++y)
+    {
+        for(x = 0; x<width; ++x)
+        {
+            if(0 == pixel[x])
+            {
+                break;
+            }
+        }
+
+        ++pixel;
+    }
+
+    GdipFree(buffer);
+
+    return (x==width) && (y==height);
+}
+
+#define IMG_WIDTH   (8)
+#define IMG_HEIGHT  (8)
+#define BLUE_COLOR  (0xff0000ff)
+static void test_GdipFillRectanglesOnMemoryDC(void)
+{
+    UINT x = 0;
+    UINT y = 0;
+    POINT pt = {0,0};
+    GpStatus status = 0;
+    union
+    {
+        GpBitmap *bitmap;
+        GpImage *image;
+    }src_img;
+    GpTexture *brush = NULL;
+    GpGraphics *graphics = NULL;
+
+    HDC dc = GetDC( hwnd);
+    HDC hdc = CreateCompatibleDC(dc);
+    HBITMAP bmp = CreateCompatibleBitmap(dc, IMG_WIDTH, IMG_HEIGHT);
+    HGDIOBJ old = SelectObject(hdc, bmp);
+
+    BYTE *src_img_data = GdipAlloc(IMG_WIDTH*IMG_HEIGHT*4);
+    DWORD *pixel = (DWORD *)src_img_data;
+    ok(pixel != NULL, "Expected src_img_data is valid\n");
+
+    /*Change the window origin is the key test point*/
+    SetWindowOrgEx(hdc, IMG_WIDTH/2, IMG_HEIGHT/2, &pt);
+
+    /*build a blue solid image to fill rectangle*/
+    for(y = 0; y < IMG_HEIGHT; ++y)
+    {
+        for(x = 0; x < IMG_WIDTH; ++x)
+        {
+            pixel[x] = BLUE_COLOR;
+        }
+
+        pixel += IMG_WIDTH;
+    }
+
+    status = GdipCreateBitmapFromScan0(IMG_WIDTH, IMG_HEIGHT, IMG_WIDTH*4,
+            PixelFormat32bppARGB, src_img_data, &src_img.bitmap);
+    expect(Ok, status);
+
+    status = GdipCreateTexture(src_img.image, 0, &brush);
+    expect(Ok, status);
+
+    status = GdipCreateFromHDC(hdc, &graphics);
+    ok(graphics != NULL, "Expected graphics is valid\n");
+
+    status = GdipFillRectangleI(graphics, (GpBrush*)brush, 0, 0, IMG_WIDTH*2, IMG_HEIGHT*2);
+    expect(Ok, status);
+
+    GdipDeleteBrush((GpBrush*)brush);
+    GdipDeleteGraphics(graphics);
+    GdipDisposeImage(src_img.image);
+    GdipFree(src_img_data);
+
+    ok(IsFillOK(hdc, bmp, IMG_WIDTH, IMG_HEIGHT), "Expected the whole bitmap is filled with a color!");
+
+    SelectObject(hdc, old);
+    DeleteObject(bmp);
+    DeleteDC(hdc);
+    ReleaseDC(hwnd, dc);
+}
 
 static void test_GdipFillRectangles(void)
 {
@@ -5619,6 +5730,7 @@ START_TEST(graphics)
     test_alpha_hdc();
     test_bitmapfromgraphics();
     test_GdipFillRectangles();
+    test_GdipFillRectanglesOnMemoryDC();
 
     GdiplusShutdown(gdiplusToken);
     DestroyWindow( hwnd );
-- 
1.9.4.msysgit.1

